import { EVENT_SERVICE } from './../../providers/service.providers';
import { EventService } from '../../services/event.service';
import { Component, Inject, OnInit } from '@angular/core';
import { Event } from '../../models/Event';
import { LocalizableError } from 'src/app/models/LocalizableError';
import { debounceTime } from 'rxjs/operators';
import { Subject, Observable } from 'rxjs';
import { LabelsService } from 'src/app/services/labels.service';
import { TimeDisplayConfig } from '../../../assets/config/config';
import { DatePipe } from '@angular/common';
import { MsTimeZoneIndexValues } from "../../constants/MSTimeZoneIndexValues";
import { environment } from "./../../../environments/environment";
import { DefaultDateFormats } from 'src/app/constants/DefaultDateFormats';

@Component({
    // tslint:disable-next-line:component-selector
    selector: 'home',
    templateUrl: './home.component.html',
    styleUrls: ['./home.component.scss']
})
export class HomeComponent implements OnInit {
    public allEvents: Event[];
    public filteredEvents: Event[];
    public isLoading: boolean;
    public error?: LocalizableError;
    private subject: Subject<string> = new Subject();
    private timeDisplayConfig: any;

    constructor(
        @Inject(EVENT_SERVICE) private eventService: EventService, 
        private labelsService: LabelsService, 
        private datePipe: DatePipe) {
    }

    ngOnInit(): void {
        this.timeDisplayConfig = TimeDisplayConfig;
        this.loadPublishedEvents();
        this.subject.pipe(
              debounceTime(300)
            ).subscribe(searchTextValue => {
              this.handleSearch(searchTextValue);
            });
    }

    private loadPublishedEvents() {
        this.isLoading = true;
        this.eventService.getPublishedEvents().subscribe(
            events => {
                this.allEvents = events;
                this.filteredEvents = events;
                this.isLoading = false;
            },
            (error: LocalizableError) => this.handleErrorResponse(error)
        );
    }

    private handleErrorResponse(error: LocalizableError) {
        this.error = error;
        this.isLoading = false;
    }

    public translateLabel(translationKey: string, defaultValue: string): Observable<string> {
        return this.labelsService.translateLabel(translationKey, defaultValue);
    }

    private getAreaLabel(event: Event): string{
        let date = this.getDateString(event);
        let building = event.building ? `at ${event.building.name}` : "";
        return `${event.eventName} on ${date} ${building}`;
    }

    private getDateString(event:Event): string{
        let dateSettings = this.getDateSettings();
        
        const locale = dateSettings.formatLocale ? this.labelsService.getCurrentLanguageCode() : null;
        const timeZoneString = event.timeZone && MsTimeZoneIndexValues[event.timeZone] || '';

        let startDateValue = this.datePipe.transform(event.startDate, dateSettings.rangeDateFormat, null, locale);
        let endDateValue = this.datePipe.transform(event.endDate, dateSettings.rangeDateFormat, null, locale);
        let startHourValue = this.datePipe.transform(event.startDate, dateSettings.timeFormat, null, locale);
        let endHourValue = this.datePipe.transform(event.endDate, dateSettings.timeFormat, null, locale);

        const startDate = new Date(event.startDate.toString());
        const endDate = new Date(event.endDate.toString());
        let includeTime = startDate.getFullYear() == endDate.getFullYear() &&
                          startDate.getMonth() === endDate.getMonth() && 
                          startDate.getDay() ===  endDate.getDay();
        if(includeTime)
        {
            const rangeTimeStructure =  dateSettings.rangeTimeStructure;
            return rangeTimeStructure.replace("[date]", startDateValue)
                                     .replace("[startTime]", startHourValue)
                                     .replace("[endTime]", endHourValue)
                                     .replace("[z]", timeZoneString)
        }
        
        const rangeDateStructure = dateSettings.rangeDateStructure;  
        return rangeDateStructure.replace("[startDate]", startDateValue)
                                 .replace("[startTime]", startHourValue)
                                 .replace("[endDate]", endDateValue)
                                 .replace("[endTime]", endHourValue)
                                 .replace("[z]", timeZoneString)
    }
    
    private getDateSettings() {
        let dateSettings = DefaultDateFormats;
        if (environment.dateSettings) {
            dateSettings = environment.dateSettings;
        }
        return dateSettings;
    }

    handleSearch(searchText: string) {
        if (searchText) {
            searchText = searchText.toLocaleLowerCase();
        }
        this.filteredEvents = this.allEvents.filter( (it: Event) => {
            return it.eventName.toString().toLocaleLowerCase().includes(searchText);
        });
    }

    search(searchText: string) {
        this.subject.next(searchText);
    }

    orderEvents(order: string) {
        if (order === "date-asc") {
            this.filteredEvents.sort((a,b) => {return <any>new Date(a.startDate.toString()) - <any>new Date(b.startDate.toString())});
        } else if (order === "date-desc") {
            this.filteredEvents.sort((a,b) => {return <any>new Date(b.startDate.toString()) - <any>new Date(a.startDate.toString())});
        } else if (order === "name-asc") {
            this.filteredEvents.sort((a,b) => a.eventName.localeCompare(b.eventName));
        } else if (order === "name-desc") {
            this.filteredEvents.sort((a,b) => b.eventName.localeCompare(a.eventName));
        }
    }
}
